// Logger class

package ladasha.io;

import java.io.*;
import java.util.*;
import java.text.*;

public class Logger {
  // singleton
  private static Logger instance = null;

  private Logger() {
    initialize();
  }
  
  public static Logger getInstance() {
    return getInstance(DEBUG);
  }
  
  public static Logger getInstance(int level) {
    if (instance == null) {
      instance = new Logger();
      instance.verbosity = level;
    }
    return instance;
  }
  
  // fields
  private File logfile;
  private FileWriter writer;
  private Date date = new Date();
  private SimpleDateFormat timestampformat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
  private static int verbosity;
  
  public static final int CRITICAL = 1;
  public static final int ERROR = 2;
  public static final int DEBUG = 3;
  public static final int NORMAL = 4;
  public static final int VERBOSE = 5;
  
  
  // methods
  private void initialize() {
    SimpleDateFormat dateformatYYYYMMDD = new SimpleDateFormat("yyyyMMdd");
    logfile = new File("midihumanizer-" + dateformatYYYYMMDD.format(date).toString() + ".log");
    try {
      boolean append = logfile.createNewFile(); // create new file if necessary
      writer = new FileWriter(logfile, true); // append if already exists
      writer.write("MIDI Humanizer - Logfile\n");
      log("Logging started... appending? " + append);
    } catch (Exception ex) {ex.printStackTrace();}
  }
  
  private String getTimestamp() {
    return timestampformat.format(Calendar.getInstance().getTime()).toString() + " -> ";
  }
  
  private String getStackTraceAsString(Throwable contents) {
    Writer result = new StringWriter();
    PrintWriter printWriter = new PrintWriter(result);
    contents.printStackTrace(printWriter);
    return result.toString();
  }
  
  private String getLevel(int level) {
    switch (level) {
      case CRITICAL:
        return "[CRITICAL] ";
      case ERROR:
        return "[ERROR] ";
      case DEBUG:
        return "[DEBUG] ";
      case NORMAL:
        return "[NORMAL] ";
      case VERBOSE:
        return "[VERBOSE] ";
      default:
        return "[UNKNOWN LEVEL: " + level + "] ";
    }
  }
  
  public void log(String contents) {log(DEBUG, contents);}
  public void log(int level, String contents) {
    if (level > verbosity) return;
    try {
      writer.write(getLevel(level) + getTimestamp() + contents + "\n");
      writer.flush(); // push it to disk
    } catch (Exception ex) {ex.printStackTrace();}
  }
  
  public void log(Throwable contents) {log(CRITICAL, contents);}
  public void log(int level, Throwable contents) {
    if (level > verbosity) return;
    try {
      writer.write(getLevel(level) + getTimestamp() + "Exception thrown! Details follow:\n");
      writer.write(getStackTraceAsString(contents));
      writer.flush();
      contents.printStackTrace(); // dump to standard error
    } catch (Exception ex) {ex.printStackTrace();}
  }
}

/*
class LoggerTester {
  public static void main(String[] args) throws Exception {
    Logger logger = Logger.getInstance();
    logger.log("test");
    Thread.sleep(2000);
    logger.log("test2");
    logger.log(new Exception("This is a test exception."));
    System.exit(-1);
  }
}
*/